# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2021 CERN.
# Copyright (C) 2021 Northwestern University.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Records service component base classes."""

from invenio_i18n import gettext as _
from invenio_records_resources.services.files.transfer import TransferType
from invenio_records_resources.services.records.components import FilesOptionsComponent
from marshmallow import ValidationError

from .base import ServiceComponent


class DraftFilesComponent(ServiceComponent):
    """Draft files service component."""

    def __init__(self, service, *args, **kwargs):
        """Constructor."""
        super().__init__(service)
        self.files_component = FilesOptionsComponent(service)

    #
    # API
    #
    def create(self, identity, data=None, record=None, errors=None):
        """Assigns files.enabled.

        NOTE: `record` actually refers to the draft
              (this interface is used in records-resources and rdm-records)
        """
        draft = record
        enabled = data.get("files", {}).get("enabled", True)

        if draft.files.enabled != enabled:
            if not self.service.check_permission(identity, "manage_files"):
                errors.append(
                    {
                        "field": "files.enabled",
                        "messages": [
                            _("You don't have permissions to manage files options.")
                        ],
                    }
                )
                return  # exit early

        draft.files.enabled = enabled

    def update_draft(self, identity, data=None, record=None, errors=None):
        """Assigns files.enabled and warns if files are missing.

        NOTE: `record` actually refers to the draft
              (this interface is used in records-resources and rdm-records)
        """
        draft = record
        enabled = data.get("files", {}).get("enabled", True)
        default_preview = data.get("files", {}).get("default_preview")

        if draft.files.enabled != enabled:
            if not self.service.check_permission(identity, "manage_files"):
                errors.append(
                    {
                        "field": "files.enabled",
                        "messages": [
                            _("You don't have permissions to manage files options.")
                        ],
                    }
                )
                return  # exit early

        try:
            self.files_component.assign_files_enabled(draft, enabled)
        except ValidationError as e:
            errors.append({"field": "files.enabled", "messages": e.messages})
            return  # exit early

        if draft.files.enabled and not draft.files.items():
            errors.append(
                {
                    "field": "files.enabled",
                    "messages": [
                        _(
                            "Missing uploaded files. To disable files for "
                            "this record please mark it as metadata-only."
                        )
                    ],
                }
            )

        try:
            self.files_component.assign_files_default_preview(
                draft,
                default_preview,
            )
        except ValidationError as e:
            errors.append({"field": "files.default_preview", "messages": e.messages})

    def edit(self, identity, draft=None, record=None):
        """Edit callback."""
        if draft.bucket is None:
            # Happens, when a soft-deleted draft is un-deleted.
            draft["files"] = {"enabled": True}
            draft.files.create_bucket()
        draft.files.copy(record.files)

    def new_version(self, identity, draft=None, record=None):
        """New version callback."""
        # We don't copy files from the previous version, but instead allow
        # users to import the files.
        draft.files.enabled = record.files.enabled

    def _publish_new(self, identity, draft, record):
        """Action when publishing a new draft."""
        # For unpublished drafts (new and new version), we move the draft
        # bucket from the draft to the record (instead of creating a new, and
        # deleting one). For consistency, we keep a bucket for all records
        # independently of if they have files enabled or not.
        record.files.set_bucket(draft.bucket)
        record.files.copy(draft.files, copy_obj=False)

        # Lock the bucket
        # TODO: Make the locking step optional in the future (so
        # instances can potentially allow files changes if desired).
        record.files.lock()

        # Cleanup
        if draft.files.enabled:
            draft.files.delete_all(remove_obj=False)
        draft.files.unset_bucket()

    def _publish_edit(self, identity, draft, record):
        """Action when publishing a edit to an existing record."""
        # TODO: For published records, we should sync changes from the
        # draft bucket to the record bucket, so that an instance could
        # potentially allow a user to update files. For now, sync() only
        # changes the default_preview and order
        record.files.sync(draft.files)

        # Teardown the bucket and files created in edit().
        if draft.files.enabled:
            draft.files.delete_all()
        draft.files.remove_bucket(force=True)

    def publish(self, identity, draft=None, record=None):
        """Copy bucket and files to record."""
        if draft.files.enabled and draft.files.bucket:
            if not draft.files.items():
                raise ValidationError(
                    _(
                        "Missing uploaded files. To disable files for "
                        "this record please mark it as metadata-only."
                    ),
                    field_name="files.enabled",
                )
        if draft.files.enabled:
            for file_record in draft.files.values():
                if not TransferType(file_record.file.storage_class).is_completed:
                    raise ValidationError(
                        _(
                            "One or more files have not completed their transfer, please wait."
                        ),
                        field_name="files",
                    )

        if record.bucket_id:
            self._publish_edit(identity, draft, record)
        else:
            self._publish_new(identity, draft, record)

    def delete_draft(self, identity, draft=None, record=None, force=False):
        """Delete files associated with a draft.

        :param force: If force is True, it means that the draft is being force
            deleted instead of soft deleted (i.e. an unpublished draft).
        """
        if draft.files.enabled:
            draft.files.delete_all(draft)
        draft.files.remove_bucket(force=True)

    def import_files(self, identity, draft=None, record=None):
        """Import files handler."""
        if not draft.files.enabled:
            raise ValidationError(
                _("Files support must be enabled."), field_name="files.enabled"
            )

        if draft.files.items():
            raise ValidationError(
                _("Please remove all files first."), field_name="files.enabled"
            )

        if not record.files.enabled and not record.files.bucket:
            raise ValidationError(
                _("The record has no files."), field_name="files.enabled"
            )

        # Copy over the files
        draft.files.copy(record.files)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""

import pytest
from flask_principal import Identity, Need, UserNeed
from invenio_app.factory import create_api as _create_api
from invenio_records_resources.services.files import FileService
from mock_module.service import DraftFileServiceConfig, FileServiceConfig

pytest_plugins = ("celery.contrib.pytest",)


@pytest.fixture(scope="module")
def app_config(app_config):
    """Override pytest-invenio app_config fixture.

    Needed to set the fields on the custom fields schema.
    """
    app_config["RECORDS_RESOURCES_FILES_ALLOWED_DOMAINS"] = [
        "inveniordm.test",
    ]
    app_config["FILES_REST_STORAGE_CLASS_LIST"] = {
        "L": "Local",
        "F": "Fetch",
        "R": "Remote",
    }

    app_config["FILES_REST_DEFAULT_STORAGE_CLASS"] = "L"

    return app_config


@pytest.fixture(scope="module")
def extra_entry_points():
    """Extra entry points to load the mock_module features."""
    return {
        "invenio_db.model": [
            "mock_module = mock_module.models",
        ],
        "invenio_jsonschemas.schemas": [
            "mock_module = mock_module.jsonschemas",
        ],
        "invenio_search.mappings": [
            "draftsresources = mock_module.mappings",
        ],
    }


@pytest.fixture(scope="module")
def create_app(instance_path, entry_points):
    """Application factory fixture."""
    return _create_api


@pytest.fixture()
def input_data():
    """Input data (as coming from the view layer).

    Most tests don't care about files, so files are disabled in the fixture.
    """
    return {"metadata": {"title": "Test"}, "files": {"enabled": False}}


@pytest.fixture(scope="module")
def file_service():
    """File service fixture."""
    return FileService(FileServiceConfig)


@pytest.fixture(scope="module")
def draft_file_service():
    """File service fixture."""
    return FileService(DraftFileServiceConfig)


@pytest.fixture()
def identity_simple():
    """Simple identity fixture."""
    i = Identity(1)
    i.provides.add(UserNeed(1))
    i.provides.add(Need(method="system_role", value="any_user"))
    return i



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
# Copyright (C) 2020-2021 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Service tests.

Test to add:
- Read a tombstone page
- Read with missing permissions
- Read with missing pid
"""

from io import BytesIO
from unittest.mock import MagicMock, patch

import pytest
from invenio_files_rest.errors import InvalidOperationError
from invenio_files_rest.models import Bucket, FileInstance, ObjectVersion
from invenio_records_resources.services.files.transfer import TransferType
from marshmallow.exceptions import ValidationError
from mock_module.models import DraftMetadata, FileDraftMetadata, FileRecordMetadata
from mock_module.permissions import PermissionPolicy
from mock_module.service import ServiceConfig

from invenio_drafts_resources.services import RecordService


#
# Fixtures
#
@pytest.fixture()
def input_data(input_data):
    """Enable files."""
    input_data["files"]["enabled"] = True
    return input_data


@pytest.fixture(scope="module")
def service(file_service, draft_file_service):
    """Service instance."""
    return RecordService(
        ServiceConfig,
        files_service=file_service,
        draft_files_service=draft_file_service,
    )


@pytest.fixture(scope="module")
def base_app(base_app, service, file_service, draft_file_service):
    """Application factory fixture."""
    registry = base_app.extensions["invenio-records-resources"].registry
    registry.register(service, service_id="mock-records-service")
    registry.register(file_service, service_id="mock-files-service")
    registry.register(draft_file_service, service_id="mock-draftfiles-service")
    yield base_app


#
# Helpers
#
def add_file_to_draft(draft_file_service, draft_id, file_id, identity):
    """Add a file to the record."""
    draft_file_service.init_files(identity, draft_id, data=[{"key": file_id}])
    draft_file_service.set_file_content(
        identity, draft_id, file_id, BytesIO(b"test file content")
    )
    result = draft_file_service.commit_file(identity, draft_id, file_id)
    return result


def assert_counts(buckets=0, objs=0, fileinstances=0, filedrafts=0, filerecords=0):
    """Helper to assert counts of file related tables."""
    assert FileDraftMetadata.query.count() == filedrafts
    assert FileRecordMetadata.query.count() == filerecords
    assert Bucket.query.count() == buckets
    assert ObjectVersion.query.count() == objs
    assert FileInstance.query.count() == fileinstances


#
# Files tests (ensure proper management of all related objects)
#
def test_create_delete_draft(app, db, service, input_data, identity_simple):
    """Test creation and deletion of a draft."""
    assert_counts(buckets=0, objs=0, fileinstances=0, filedrafts=0)

    # Create draft
    draft = service.create(identity_simple, input_data)
    assert_counts(buckets=1, objs=0, fileinstances=0, filedrafts=0)

    # Add file
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=1)

    # Delete draft
    service.delete_draft(identity_simple, draft.id)
    assert_counts(buckets=0, objs=0, fileinstances=1, filedrafts=0)


def test_create_publish(app, db, service, input_data, identity_simple):
    """Test creation and publish of a draft."""
    # Create draft and file
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=1)

    # Publish
    record = service.publish(identity_simple, draft.id)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=0, filerecords=1)
    assert record._record.bucket.locked is True


def test_edit_delete(app, db, service, input_data, identity_simple, monkeypatch):
    """Test edit with delete of a published draft."""
    # Create and publish
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    record = service.publish(identity_simple, draft.id)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=0, filerecords=1)

    # Edit draft (when soft-deleted draft record exists)
    draft = service.edit(identity_simple, record.id)
    assert_counts(buckets=2, objs=2, fileinstances=1, filedrafts=1, filerecords=1)
    assert record._record.bucket.locked is True
    assert draft._record.bucket.locked is False

    # Delete draft
    draft = service.delete_draft(identity_simple, draft.id)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=0, filerecords=1)

    # Cleanup deleted drafts.
    c = DraftMetadata.query.filter(DraftMetadata.is_deleted == True).delete()  # noqa
    assert c == 1
    db.session.commit()

    # Search engine will complain if we try we don't wait a minute, so disable
    # the indexer.
    monkeypatch.setattr(service.config, "indexer_cls", MagicMock())

    # Edit draft (when no soft-deleted draft record exists)
    draft = service.edit(identity_simple, record.id)
    assert_counts(buckets=2, objs=2, fileinstances=1, filedrafts=1, filerecords=1)

    # Delete draft
    draft = service.delete_draft(identity_simple, draft.id)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=0, filerecords=1)


def test_edit_publish(app, db, service, input_data, identity_simple, monkeypatch):
    """Test edit and publish."""
    # Create, publish and edit draft.
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    record = service.publish(identity_simple, draft.id)
    draft = service.edit(identity_simple, record.id)
    assert_counts(buckets=2, objs=2, fileinstances=1, filedrafts=1, filerecords=1)

    # Publish the edits
    record = service.publish(identity_simple, draft.id)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=0, filerecords=1)

    # Cleanup deleted drafts.
    c = DraftMetadata.query.filter(DraftMetadata.is_deleted == True).delete()  # noqa
    assert c == 1
    db.session.commit()

    # engine will complain if we try we don't wait a minute, so disable
    # the indexer.
    monkeypatch.setattr(service.config, "indexer_cls", MagicMock())

    # Edit
    draft = service.edit(identity_simple, record.id)
    assert_counts(buckets=2, objs=2, fileinstances=1, filedrafts=1, filerecords=1)

    # Publish
    record = service.publish(identity_simple, draft.id)
    assert_counts(buckets=1, objs=1, fileinstances=1, filedrafts=0, filerecords=1)


def test_new_version(app, db, service, input_data, identity_simple, monkeypatch):
    """Test new version."""
    # Create and publish
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    service.publish(identity_simple, draft.id)

    # New version
    draft = service.new_version(identity_simple, draft.id)
    assert_counts(buckets=2, objs=1, fileinstances=1, filedrafts=0, filerecords=1)

    # Add file
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    assert_counts(buckets=2, objs=2, fileinstances=2, filedrafts=1, filerecords=1)

    # Publish
    service.publish(identity_simple, draft.id)
    assert_counts(buckets=2, objs=2, fileinstances=2, filedrafts=0, filerecords=2)


def test_new_version_delete(app, db, service, input_data, identity_simple):
    """Test new version."""
    # Create, publish, new_version, add_file
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    service.publish(identity_simple, draft.id)
    draft = service.new_version(identity_simple, draft.id)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    assert_counts(buckets=2, objs=2, fileinstances=2, filedrafts=1, filerecords=1)

    # Delete new version
    service.delete_draft(identity_simple, draft.id)
    assert_counts(buckets=1, objs=1, fileinstances=2, filedrafts=0, filerecords=1)


#
# Test import files
#
def test_import_files(app, db, service, input_data, identity_simple):
    """Test new version."""
    # Create, publish, new_version
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    service.publish(identity_simple, draft.id)
    draft = service.new_version(identity_simple, draft.id)
    assert_counts(buckets=2, objs=1, fileinstances=1, filedrafts=0, filerecords=1)

    # Import files
    service.import_files(identity_simple, draft.id)
    assert_counts(buckets=2, objs=2, fileinstances=1, filedrafts=1, filerecords=1)


def test_import_files_disabled(app, db, service, input_data, identity_simple):
    """Test new version."""
    # Create, publish, new_version
    input_data["files"]["enabled"] = False
    draft = service.create(identity_simple, input_data)
    service.publish(identity_simple, draft.id)
    draft = service.new_version(identity_simple, draft.id)
    assert_counts(buckets=2, objs=0, fileinstances=0, filedrafts=0, filerecords=0)

    with pytest.raises(ValidationError):
        service.import_files(identity_simple, draft.id)


#
# Default preview
#
def test_edit_publish_default_preview(app, service, input_data, identity_simple):
    """Test edit and publish."""
    # Create, publish and edit draft.
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, "test", identity_simple)
    record = service.publish(identity_simple, draft.id)
    draft = service.edit(identity_simple, record.id)
    assert "default_preview" not in draft.data["files"]

    # Set the default preview
    data = draft.to_dict()
    data["files"]["default_preview"] = "test"
    draft = service.update_draft(identity_simple, draft.id, data)
    # TODO: fails because it needs schema to load the value.
    assert draft.data["files"]["default_preview"] == "test"

    # Publish change
    record = service.publish(identity_simple, draft.id)
    assert record.data["files"]["default_preview"] == "test"


def test_update_draft_set_default_file_preview(
    app, location, service, identity_simple, input_data
):
    default_file = "file.txt"

    # Create and add file
    draft = service.create(identity_simple, input_data)
    add_file_to_draft(service.draft_files, draft.id, default_file, identity_simple)

    # Update draft to set default preview
    input_data["files"] = {"enabled": True, "default_preview": default_file}
    draft = service.update_draft(identity_simple, draft.id, input_data)

    # Default preview should have been updated
    assert draft.data["files"] == {"enabled": True, "default_preview": default_file}
    assert default_file == draft._record.files.default_preview


def test_update_draft_set_default_file_preview_reports_error(
    app, location, service, identity_simple, input_data
):
    # Create and add file
    draft = service.create(identity_simple, input_data)
    default_file = "file.txt"
    add_file_to_draft(service.draft_files, draft.id, default_file, identity_simple)

    # Fail to set default preview to invalid file
    input_data["files"] = {"enabled": True, "default_preview": "inexisting_file.txt"}
    draft = service.update_draft(identity_simple, draft.id, input_data)

    # Default preview should NOT be set
    assert draft.errors[0]["field"] == "files.default_preview"
    assert draft.errors[0]["messages"]
    assert draft.data["files"] == {"enabled": True}


@patch("invenio_records_resources.services.files.transfer.fetch_file")
def test_publish_with_fetch_files(
    p_fetch_file, app, service, draft_file_service, input_data, identity_simple
):
    """Tests wether it is possible to submit a record if the file isn't fully downloaded."""
    draft = service.create(identity_simple, input_data)  # 1
    file_to_initialise = [
        {
            "key": "article.txt",
            "uri": "https://inveniordm.test/files/article.txt",
            "storage_class": "F",
        }
    ]

    files = draft_file_service.init_files(
        identity_simple, draft.id, data=file_to_initialise
    )

    for file_record in files.entries:
        assert file_record["storage_class"] == TransferType.FETCH

    with pytest.raises(ValidationError):
        service.publish(identity_simple, draft.id)


#
# Files validation tests
#
def test_missing_files(app, service, identity_simple, input_data):
    # Test files.enabled = True when no files
    draft = service.create(identity_simple, input_data)
    draft = service.update_draft(identity_simple, draft.id, input_data)

    # Files should be enabled
    assert draft.data["files"]["enabled"] is True

    # Missing files should be reported
    assert draft.errors[0]["field"] == "files.enabled"
    assert "Missing uploaded files." in draft.errors[0]["messages"][0]


def test_disable_files_error(app, service, identity_simple, input_data):
    draft = service.create(identity_simple, input_data)
    assert draft.data["files"]["enabled"] is True

    # Add a file
    add_file_to_draft(service.draft_files, draft.id, "file.txt", identity_simple)

    # Try to set files.enabled = False
    input_data = draft.data
    input_data["files"] = {"enabled": False}
    draft = service.update_draft(identity_simple, draft.id, input_data)

    # Should not be possible to disable files, when files exists
    assert draft.data["files"]["enabled"] is True

    # Error should be reported
    assert draft.errors[0]["field"] == "files.enabled"
    assert "delete all" in draft.errors[0]["messages"][0]


def test_fail_to_publish_draft_with_no_files(app, service, identity_simple, input_data):
    input_data["files"] = {"enabled": True}
    draft = service.create(identity_simple, input_data)

    with pytest.raises(ValidationError) as e:
        service.publish(identity_simple, draft.id)

    assert e.value.field_name == "files.enabled"
    files_missing_msg = e.value.messages
    assert files_missing_msg is not None


def test_fail_to_add_files_to_draft_with_files_disabled(
    app, service, identity_simple, input_data
):
    # NOTE: It is impossible to publish a draft with files but
    #       files.enabled = False, because can't set files.enabled = False
    #       in that case (as seen in test above) and files can't be
    #       added if files.enabled = False as we confirm below:
    input_data["files"]["enabled"] = False
    draft = service.create(identity_simple, input_data)

    with pytest.raises(InvalidOperationError):
        add_file_to_draft(service.draft_files, draft.id, "file.txt", identity_simple)


def test_manage_files_permissions(
    app, service, identity_simple, input_data, monkeypatch
):
    class ManageFilesPermissionPolicy(PermissionPolicy):
        can_manage_files = []

    monkeypatch.setattr(
        service.config,
        "permission_policy_cls",
        ManageFilesPermissionPolicy,
    )

    # Not modifying files options (i.e. sending `files.enabled: true`) doesn't error
    draft = service.create(identity_simple, input_data)
    assert not draft.errors

    # Modyfing/disabling files should return an error
    input_data["files"]["enabled"] = False
    draft = service.create(identity_simple, input_data)
    assert draft.errors[0]["field"] == "files.enabled"
    assert draft.errors[0]["messages"] == [
        "You don't have permissions to manage files options.",
    ]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2021 CERN.
# Copyright (C) 2021 Northwestern University.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Records service component base classes."""

from invenio_records_resources.services.records.components import ServiceComponent


class PIDComponent(ServiceComponent):
    """Service component for PID registraion."""

    def publish(self, identity, draft=None, record=None):
        """Register persistent identifiers when publishing."""
        if not record.is_published:
            record.register()

    def delete_draft(self, identity, draft=None, record=None, force=False):
        """Unregister persistent identifiers for unpublished drafts."""
        if force:
            draft.__class__.pid.session_merge(draft)
            draft.pid.delete()
            draft.parent.__class__.pid.session_merge(draft.parent)
            draft.parent.pid.delete()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Service tasks tests."""

import pytest

from invenio_drafts_resources.services.records.tasks import cleanup_drafts


#
# Fixtures
#
@pytest.fixture()
def input_data(input_data):
    """Enable files."""
    input_data["files"]["enabled"] = False
    return input_data


@pytest.fixture(scope="module")
def base_app(base_app, service):
    """Application factory fixture."""
    registry = base_app.extensions["invenio-records-resources"].registry
    registry.register(service, service_id="records")
    yield base_app


def test_hard_delete_soft_deleted_task(app, service, identity_simple, input_data):
    draft = service.create(identity_simple, input_data)
    service.publish(identity_simple, draft.id)
    draft_model = service.draft_cls.model_cls

    assert (
        len(draft_model.query.filter(draft_model.is_deleted == True).all()) == 1  # noqa
    )
    cleanup_drafts(seconds=0)

    assert (
        len(draft_model.query.filter(draft_model.is_deleted == True).all()) == 0  # noqa
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Record schema."""

from marshmallow import Schema, fields, validate
from marshmallow.utils import get_value
from marshmallow_utils.fields import SanitizedUnicode

from invenio_drafts_resources.services.records.schema import (
    RecordSchema as BaseRecordSchema,
)


class MetadataSchema(Schema):
    """Basic metadata schema class."""

    title = fields.Str(required=True, validate=validate.Length(min=3))


class FilesOptionsSchema(Schema):
    """Basic files options schema class."""

    enabled = fields.Bool(missing=True)
    # allow unsetting
    default_preview = SanitizedUnicode(allow_none=True)

    def get_attribute(self, obj, attr, default):
        """Override how attributes are retrieved when dumping.

        NOTE: We have to access by attribute because although we are loading
              from an external pure dict, but we are dumping from a data-layer
              object whose fields should be accessed by attributes and not
              keys. Access by key runs into FilesManager key access protection
              and raises.
        """
        value = getattr(obj, attr, default)

        if attr == "default_preview" and not value:
            return default

        return value


class RecordSchema(BaseRecordSchema):
    """Schema for records in JSON."""

    metadata = fields.Nested(MetadataSchema)
    files = fields.Nested(FilesOptionsSchema)

    def get_attribute(self, obj, attr, default):
        """Override how attributes are retrieved when dumping.

        NOTE: We have to access by attribute because although we are loading
              from an external pure dict, but we are dumping from a data-layer
              object whose fields should be accessed by attributes and not
              keys. Access by key runs into FilesManager key access protection
              and raises.
        """
        if attr == "files":
            return getattr(obj, attr, default)
        else:
            return get_value(obj, attr, default)
